@@ -1,5 +1,5 @@
 /*
-  Copyright 2011 David Robillard <http://drobilla.net>
+  Copyright 2011-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,13 +14,11 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <assert.h>
-#include <stdbool.h>
+#include "serd_internal.h"
+
 #include <stdlib.h>
 #include <string.h>
 
-#include "serd_internal.h"
-
 typedef struct {
 	SerdNode name;
 	SerdNode uri;
@@ -33,22 +31,16 @@ struct SerdEnvImpl {
 	SerdURI     base_uri;
 };
 
-SERD_API
 SerdEnv*
 serd_env_new(const SerdNode* base_uri)
 {
-	SerdEnv* env = malloc(sizeof(struct SerdEnvImpl));
-	env->prefixes      = NULL;
-	env->n_prefixes    = 0;
-	env->base_uri_node = SERD_NODE_NULL;
-	env->base_uri      = SERD_URI_NULL;
-	if (base_uri) {
+	SerdEnv* env = (SerdEnv*)calloc(sizeof(struct SerdEnvImpl), 1);
+	if (env && base_uri) {
 		serd_env_set_base_uri(env, base_uri);
 	}
 	return env;
 }
 
-SERD_API
 void
 serd_env_free(SerdEnv* env)
 {
@@ -61,24 +53,28 @@ serd_env_free(SerdEnv* env)
 	free(env);
 }
 
-SERD_API
 const SerdNode*
 serd_env_get_base_uri(const SerdEnv* env,
                       SerdURI*       out)
 {
-	*out = env->base_uri;
+	if (out) {
+		*out = env->base_uri;
+	}
 	return &env->base_uri_node;
 }
 
-SERD_API
 SerdStatus
 serd_env_set_base_uri(SerdEnv*        env,
-                      const SerdNode* uri_node)
+                      const SerdNode* uri)
 {
+	if (!env || !uri) {
+		return SERD_ERR_BAD_ARG;
+	}
+
 	// Resolve base URI and create a new node and URI for it
 	SerdURI  base_uri;
 	SerdNode base_uri_node = serd_node_new_uri_from_node(
-		uri_node, &env->base_uri, &base_uri);
+		uri, &env->base_uri, &base_uri);
 
 	if (base_uri_node.buf) {
 		// Replace the current base URI
@@ -111,39 +107,34 @@ serd_env_add(SerdEnv*        env,
              const SerdNode* name,
              const SerdNode* uri)
 {
-	assert(name && uri);
-	assert(name->n_bytes == strlen((const char*)name->buf));
 	SerdPrefix* const prefix = serd_env_find(env, name->buf, name->n_bytes);
 	if (prefix) {
 		SerdNode old_prefix_uri = prefix->uri;
 		prefix->uri = serd_node_copy(uri);
 		serd_node_free(&old_prefix_uri);
 	} else {
-		env->prefixes = realloc(env->prefixes,
-		                        (++env->n_prefixes) * sizeof(SerdPrefix));
+		env->prefixes = (SerdPrefix*)realloc(
+			env->prefixes, (++env->n_prefixes) * sizeof(SerdPrefix));
 		env->prefixes[env->n_prefixes - 1].name = serd_node_copy(name);
 		env->prefixes[env->n_prefixes - 1].uri  = serd_node_copy(uri);
 	}
 }
 
-SERD_API
 SerdStatus
 serd_env_set_prefix(SerdEnv*        env,
                     const SerdNode* name,
-                    const SerdNode* uri_node)
+                    const SerdNode* uri)
 {
-	if (serd_uri_string_has_scheme(uri_node->buf)) {
+	if (!name->buf || uri->type != SERD_URI) {
+		return SERD_ERR_BAD_ARG;
+	} else if (serd_uri_string_has_scheme(uri->buf)) {
 		// Set prefix to absolute URI
-		serd_env_add(env, name, uri_node);
+		serd_env_add(env, name, uri);
 	} else {
 		// Resolve relative URI and create a new node and URI for it
 		SerdURI  abs_uri;
 		SerdNode abs_uri_node = serd_node_new_uri_from_node(
-			uri_node, &env->base_uri, &abs_uri);
-
-		if (!abs_uri_node.buf) {
-			return SERD_ERR_BAD_ARG;
-		}
+			uri, &env->base_uri, &abs_uri);
 
 		// Set prefix to resolved (absolute) URI
 		serd_env_add(env, name, &abs_uri_node);
@@ -152,7 +143,6 @@ serd_env_set_prefix(SerdEnv*        env,
 	return SERD_SUCCESS;
 }
 
-SERD_API
 SerdStatus
 serd_env_set_prefix_from_strings(SerdEnv*       env,
                                  const uint8_t* name,
@@ -161,51 +151,13 @@ serd_env_set_prefix_from_strings(SerdEnv*       env,
 	const SerdNode name_node = serd_node_from_string(SERD_LITERAL, name);
 	const SerdNode uri_node  = serd_node_from_string(SERD_URI, uri);
 
-	SerdStatus ret = serd_env_set_prefix(env, &name_node, &uri_node);
-
-	return ret;
+	return serd_env_set_prefix(env, &name_node, &uri_node);
 }
 
-static inline bool
-is_nameStartChar(const uint8_t c)
-{
-	// TODO: not strictly correct (see reader.c read_nameStartChar)
-	return (c >= 'A' && c <= 'Z') || c == '_' || (c >= 'a' && c <= 'z');
-}
-
-static inline bool
-is_nameChar(const uint8_t c)
-{
-	// TODO: 0x300-0x036F | 0x203F-0x2040 (see reader.c read_nameChar)
-	return is_nameStartChar(c) || c == '-'
-		|| (c >= '0' && c <= '9') || c == 0xB7;
-}
-
-/**
-   Return true iff @c buf is a valid Turtle name.
-*/
-static inline bool
-is_name(const uint8_t* buf,
-        size_t         len)
-{
-	if (!is_nameStartChar(buf[0])) {
-		return false;
-	}
-
-	for (size_t i = 1; i < len; ++i) {
-		if (!is_nameChar(buf[i])) {
-			return false;
-		}
-	}
-
-	return true;
-}
-
-SERD_API
 bool
 serd_env_qualify(const SerdEnv*  env,
                  const SerdNode* uri,
-                 SerdNode*       prefix_name,
+                 SerdNode*       prefix,
                  SerdChunk*      suffix)
 {
 	for (size_t i = 0; i < env->n_prefixes; ++i) {
@@ -214,12 +166,9 @@ serd_env_qualify(const SerdEnv*  env,
 			if (!strncmp((const char*)uri->buf,
 			             (const char*)prefix_uri->buf,
 			             prefix_uri->n_bytes)) {
-				*prefix_name = env->prefixes[i].name;
+				*prefix = env->prefixes[i].name;
 				suffix->buf = uri->buf + prefix_uri->n_bytes;
 				suffix->len = uri->n_bytes - prefix_uri->n_bytes;
-				if (!is_name(suffix->buf, suffix->len)) {
-					continue;
-				}
 				return true;
 			}
 		}
@@ -227,65 +176,63 @@ serd_env_qualify(const SerdEnv*  env,
 	return false;
 }
 
-SERD_API
 SerdStatus
 serd_env_expand(const SerdEnv*  env,
-                const SerdNode* qname,
+                const SerdNode* curie,
                 SerdChunk*      uri_prefix,
                 SerdChunk*      uri_suffix)
 {
-	const uint8_t* const colon = memchr(qname->buf, ':', qname->n_bytes + 1);
-	if (!colon) {
-		return SERD_ERR_BAD_ARG;  // Illegal qname
+	const uint8_t* const colon = (const uint8_t*)memchr(
+		curie->buf, ':', curie->n_bytes + 1);
+	if (curie->type != SERD_CURIE || !colon) {
+		return SERD_ERR_BAD_ARG;
 	}
 
-	const size_t            name_len = colon - qname->buf;
-	const SerdPrefix* const prefix   = serd_env_find(env, qname->buf, name_len);
+	const size_t            name_len = colon - curie->buf;
+	const SerdPrefix* const prefix   = serd_env_find(env, curie->buf, name_len);
 	if (prefix) {
 		uri_prefix->buf = prefix->uri.buf;
 		uri_prefix->len = prefix->uri.n_bytes;
 		uri_suffix->buf = colon + 1;
-		uri_suffix->len = qname->n_bytes - (colon - qname->buf) - 1;
+		uri_suffix->len = curie->n_bytes - (colon - curie->buf) - 1;
 		return SERD_SUCCESS;
 	}
-	return SERD_ERR_NOT_FOUND;
+	return SERD_ERR_BAD_CURIE;
 }
 
-SERD_API
 SerdNode
 serd_env_expand_node(const SerdEnv*  env,
                      const SerdNode* node)
 {
-	if (node->type == SERD_CURIE) {
+	switch (node->type) {
+	case SERD_CURIE: {
 		SerdChunk prefix;
 		SerdChunk suffix;
-		serd_env_expand(env, node, &prefix, &suffix);
-		SerdNode ret = { NULL,
-		                 prefix.len + suffix.len + 1,
-		                 prefix.len + suffix.len,  // FIXME: UTF-8
-		                 0,
-		                 SERD_URI };
-		ret.buf = malloc(ret.n_bytes + 1);
-		snprintf((char*)ret.buf, ret.n_bytes + 1,
-		         "%s%s", prefix.buf, suffix.buf);
+		if (serd_env_expand(env, node, &prefix, &suffix)) {
+			return SERD_NODE_NULL;
+		}
+		const size_t len = prefix.len + suffix.len;
+		uint8_t*     buf = (uint8_t*)malloc(len + 1);
+		SerdNode     ret = { buf, len, 0, 0, SERD_URI };
+		snprintf((char*)buf, len + 1, "%s%s", prefix.buf, suffix.buf);
+		ret.n_chars = serd_strlen(buf, NULL, NULL);
 		return ret;
-	} else if (node->type == SERD_URI) {
+	}
+	case SERD_URI: {
 		SerdURI ignored;
 		return serd_node_new_uri_from_node(node, &env->base_uri, &ignored);
-	} else {
+	}
+	default:
 		return SERD_NODE_NULL;
 	}
 }
 
-SERD_API
 void
 serd_env_foreach(const SerdEnv* env,
                  SerdPrefixSink func,
                  void*          handle)
 {
 	for (size_t i = 0; i < env->n_prefixes; ++i) {
-		func(handle,
-		     &env->prefixes[i].name,
-		     &env->prefixes[i].uri);
+		func(handle, &env->prefixes[i].name, &env->prefixes[i].uri);
 	}
 }
